<?php

namespace Controller;

use Exception\CustomException;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO;
use \Util\Messages;
class ProductDeploy extends Simple
{
    /** @var \Service\ProductDeploy $productDeployService */
    protected $productDeployService;

    public function __construct(\Service\ProductDeploy $productDeployService) {
        $this->productDeployService = $productDeployService;
    }

    public function findCustomerProducts(Request\Filter $request, Response $response) {
        try {
            $conditions       = $request->getFilterCriteria()->getConditions();
            $page       = $request->getFilterCriteria()->getPage();
            $pageSize   = $request->getFilterCriteria()->getPageSize();
            $productType      = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_TYPE', true);
            $customerProducts = $this->productDeployService->findCustomerProducts($conditions, $productType, $page, $pageSize);
            $response->addDataSet(new DataSet('findCustomerProducts', $customerProducts));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveCustomerProduct(Request\DataSet $request, Response $response) {
        try {
            $customerProduct = $request->getDataSet()->getRows()[0];
            $productId       = empty($customerProduct['NREXTERNAL_PRODUCT_ID']) ? null : $customerProduct['NREXTERNAL_PRODUCT_ID'];
            $versionCheck    = empty($customerProduct['VERSION_CHECK_TYPE']) ? null : $customerProduct['VERSION_CHECK_TYPE'];
            $urlWebService   = empty($customerProduct['URL_WEBSERVICE']) ? null : $customerProduct['URL_WEBSERVICE'];
            $productId       = $this->productDeployService->saveCustomerProduct($productId, $versionCheck, $urlWebService);
            $response->addDataSet(new DataSet('savedProduct', array($productId)));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findCustomerServers(Request\Filter $request, Response $response) {
        try {
            $conditions     = $request->getFilterCriteria()->getConditions();
            $productId      = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $customerServer = $this->productDeployService->findCustomerServers($productId, $conditions);
            $response->addDataSet(new DataSet('findCustomerServers', $customerServer));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getDeployAudit(Request\Filter $request, Response $response) {
        try {
            //
            $conditions     = $request->getFilterCriteria()->getConditions();
            $initialDate      = Utilities::getValueInFilterDataSourceByName($conditions, 'DATE_INTERVAL', true);
            $productId      = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $serverId      = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_ID', true);
            $isClient      = Utilities::getValueInFilterDataSourceByName($conditions, 'IS_CLIENT', true);

            $startDate = $initialDate['start'];
            $endDate = $initialDate['end'];

            $customerServer = $this->productDeployService->findDeployAudit($productId, $serverId, $startDate, $endDate, $isClient);
            $response->addDataSet(new DataSet('findDeployAudit', $customerServer));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getVersionAudit(Request\Filter $request, Response $response) {
        try {
            $conditions     = $request->getFilterCriteria()->getConditions();

            $productId      = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $moduleId      = Utilities::getValueInFilterDataSourceByName($conditions, 'MODULE_PRODUCT_ID', true);

            $initialFullVersion      = explode('.' ,Utilities::getValueInFilterDataSourceByName($conditions, 'INITIAL_VERSION', true));
            $finalFullVersion      = explode('.', Utilities::getValueInFilterDataSourceByName($conditions, 'FINAL_VERSION', true));
            $isClient      = Utilities::getValueInFilterDataSourceByName($conditions, 'IS_CLIENT', true);

            $initialVersion = $initialFullVersion[0];
            $finalVersion = $finalFullVersion[0];

            $initialRevision = !isset($initialFullVersion[1]) ? 0 : $initialFullVersion[1];
            $finalRevision = !isset($finalFullVersion[1]) ? 99999 : $finalFullVersion[1];

            $initialBuild = !isset($initialFullVersion[2]) ? 0 : $initialFullVersion[2];
            $finalBuild = !isset($finalFullVersion[2]) ? 99999 : $finalFullVersion[2];
            $customerServer = $this->productDeployService->findVersionAudit($productId, $moduleId,  $isClient, $initialVersion, $finalVersion, $initialRevision,$finalRevision,$initialBuild,$finalBuild);
            $response->addDataSet(new DataSet('findDeployAudit', $customerServer));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveCustomerServer(Request\DataSet $request, Response $response) {
        try {
            $customerServer  = $request->getDataSet()->getRows()[0];
            $productServerId = empty($customerServer['PRODUCT_SERVER_ID']) ? null : $customerServer['PRODUCT_SERVER_ID'];
            $path            = empty($customerServer['PATH']) ? null : $customerServer['PATH'];
            $productId       = empty($customerServer['PRODUCT_ID']) ? null : $customerServer['PRODUCT_ID'];
            $packageId       = empty($customerServer['PACKAGE_ID']) ? null : $customerServer['PACKAGE_ID'];
            $serverId        = empty($customerServer['SERVER_ID']) ? null : $customerServer['SERVER_ID'];
            $backupLimit     = empty($customerServer['BACKUP_LIMIT']) ? null : $customerServer['BACKUP_LIMIT'];
            $execScriptsAuto = empty($customerServer['EXEC_SCRIPTS_AUTOMATIC']) ? null : $customerServer['EXEC_SCRIPTS_AUTOMATIC'];
            $productVersionId  = $this->productDeployService->findLatestProductVersion($productId);
            $modulesIds = empty($productVersionId) ? null : $this->productDeployService->findModulesConfiguration($productVersionId[0]['PRODUCT_VERSION_ID'], $productId);
            $serverId        = $this->productDeployService->saveCustomerServer($productServerId, $path, $packageId,
                $productId, $serverId, $backupLimit, $execScriptsAuto, $modulesIds);
            $response->addDataSet(new DataSet('savedServer', array($serverId)));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function removeCustomerServer(Request\DataSet $request, Response $response) {
        try {
            $customerServer  = $request->getDataSet()->getRows()[0];
            $productServerId = empty($customerServer['PRODUCT_SERVER_ID']) ? null : $customerServer['PRODUCT_SERVER_ID'];
            $this->productDeployService->removeCustomerServer($productServerId);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getAvailableVersionInfo(Request\Filter $request, Response $response) {
        try {
            $conditions       = $request->getFilterCriteria()->getConditions();
            $modulesVersion   = Utilities::getValueInFilterDataSourceByName($conditions, 'MODULES_VERSION', true);
            $productId        = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $productVersionId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_VERSION_ID', true);
            $previousDeploy   = Utilities::getValueInFilterDataSourceByName($conditions, 'PREVIOUS_DEPLOY', true);
            $serverId   = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_ID', true);
            $availableVersion = $this->productDeployService->getAvailableVersionInfo($modulesVersion, $productId, $productVersionId, $previousDeploy, $serverId);
            $response->addDataSet(new DataSet('availableVersionInfo', $availableVersion));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function getActualModulesProductVersions(Request\Filter $request, Response $response) {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $modulesVersion  = Utilities::getValueInFilterDataSourceByName($conditions, 'MODULES_VERSION', true);
            $versions        = $this->productDeployService->getActualModulesProductVersions($modulesVersion);
            $response->addDataSet(new DataSet('getActualModulesProductVersions', $versions));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function getProductModules(Request\Filter $request, Response $response) {
        try {
            $conditions       = $request->getFilterCriteria()->getConditions();
            $productId        = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT', true);
            $availableVersion = $this->productDeployService->getProductModules($productId);
            $response->addDataSet(new DataSet('availableVersionInfo', $availableVersion));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function findCustomerDeploys(Request\Filter $request, Response $response) {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $productId       = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $customerDeploys = $this->productDeployService->findCustomerDeploys($productId, $conditions);
            $response->addDataSet(new DataSet('findCustomerDeploys', $customerDeploys));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findCustomerDeploySteps(Request\Filter $request, Response $response) {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $deployStepId    = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_STEP_ID', true);
            $customerDeploys = $this->productDeployService->findCustomerDeploySteps($deployStepId);
            $response->addDataSet(new DataSet('findCustomerDeploySteps', $customerDeploys));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getApplyDeployInfo(Request\Filter $request, Response $response) {
        try {
            $conditions         = $request->getFilterCriteria()->getConditions();
            $productId          = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $productServerId    = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_SERVER_ID', true);
            $productModules     = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_MODULES', true);
            $productPath        = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_PATH', true);
            $checkedModulesIds  = Utilities::getValueInFilterDataSourceByName($conditions, 'CHECKED_MODULES_IDS', true);
            $deployStepServerId = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_STEP_SERVER_ID', true);
            $mainDeployId       = Utilities::getValueInFilterDataSourceByName($conditions, 'MAIN_DEPLOY_ID', true);
            $applyDeployInfo = $this->productDeployService->getApplyDeployInfo($productId, $productServerId,
                $productModules, $productPath, $checkedModulesIds, $deployStepServerId, $mainDeployId);
            $response->addDataSet(new DataSet('applyDeployInfo',  $applyDeployInfo['PRODUCT_MODULES']));
            $response->addDataSet(new DataSet('currentUserData',  $applyDeployInfo['CURRENT_USER_DATA']));
            $response->addDataSet(new DataSet('migrationQueries', $applyDeployInfo['MIGRATION_QUERIES']));
            $response->addDataSet(new DataSet('repositoriesInfo', $applyDeployInfo['REPOSITORIES_INFO']));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function getProductDeployScripts(Request\Filter $request, Response $response) {
        try {
            $conditions       = $request->getFilterCriteria()->getConditions();
            $productId        = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $productServerId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_SERVER_ID', true);
            $productVersionId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_VERSION_ID', true);
            $serverIsWindows  = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_IS_WINDOWS', true);
            $applyDeployInfo  = $this->productDeployService->getProductDeployScripts($productId, $productServerId, $productVersionId, $serverIsWindows);
            $response->addDataSet(new DataSet('deployScripts', $applyDeployInfo));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function sendLogByEmail(Request\Dataset $request, Response $response)
    {
        $row = $request->getDataSet()->getRows();
        $row = $row[0];
        try {
            $param = $row;
            $param['LOG_Email'] = empty($param['LOG_Email']) ? null : $param['LOG_Email'];
            $logFileContents = $this->productDeployService->getLogToSend($param['LOG_FILE_NAME']);
            if ($logFileContents != ""){
                $filename = $param['LOG_FILE_NAME'];
                $emails = explode(";", $param['LOG_Email']);
                $temp = Utilities::getTemporaryDir();
                $dir = $temp.DIRECTORY_SEPARATOR.$filename;
                $this->productDeployService->sendLogEmail($emails, $filename, $dir);
                $logFileContents = utf8_encode($logFileContents);
                $response->addDataSet(new DataSet('logFileContents', [$logFileContents]));
            }
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function findProductPackages(Request\Filter $request, Response $response) {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $productId       = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $productPackages = $this->productDeployService->findProductPackages($productId, $conditions);
            $response->addDataSet(new DataSet('productPackages', $productPackages));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function inactiveDeploysRemoved(Request\Dataset $request, Response $response) {
        try {
            $deploy = $request->getDataSet()->getRows()[0];
            $currentDeployId = $deploy['CURRENT_DEPLOY_ID'];
            $productServerId = $deploy['PRODUCT_SERVER_ID'];
            $removedPaths    = $deploy['REMOVED_PATHS'];
            $backupLimit     = $deploy['BACKUP_LIMIT'];
            $productId       = $deploy['PRODUCT_ID'];
            $this->productDeployService->inactiveDeploysRemoved($productServerId, $removedPaths, $currentDeployId,
                $backupLimit, $productId);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveCustomerDeploy(Request\DataSet $request, Response $response) {
        try {
            $deploy           = $request->getDataSet()->getRows()[0];
            $deployId         = $deploy['ID'];
            $status           = $deploy['STATUS'];
            $isActive         = empty($deploy['IS_ACTIVE']) ? 'Y'    : $deploy['IS_ACTIVE'];;
            $date             = empty($deploy['DEPLOY_DATE']) ? null : $deploy['DEPLOY_DATE'];
            $pathBkp          = empty($deploy['PATH_BACKUP']) ? null : $deploy['PATH_BACKUP'];
            $productModules   = empty($deploy['PRODUCT_MODULES']) ? array() : $deploy['PRODUCT_MODULES'];
            $ticket           = empty($deploy['TICKET']) ? null : $deploy['TICKET'];
            $server           = empty($deploy['SERVER']) ? null : $deploy['SERVER'];
            $version          = empty($deploy['VERSION']) ? null : $deploy['VERSION'];
            $updateModulesIds = empty($deploy['UPDATE_MODULES_IDS']) ? [] : $deploy['UPDATE_MODULES_IDS'];
            $isUpdateProduct  = empty($deploy['IS_UPDATE_PRODUCT']) ? false : $deploy['IS_UPDATE_PRODUCT'];
            $savedDeploy = $this->productDeployService->saveDeploy($deployId, $date, $isActive, $pathBkp, $status, 
                $productModules, $ticket, $server, $version, $updateModulesIds, $isUpdateProduct);
            $response->addDataSet(new DataSet('savedDeploy', $savedDeploy));
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function updateDeployStepRemoveDeploy(Request\DataSet $request, Response $response) {
        try {
            $deploy   = $request->getDataSet()->getRows()[0];
            $deployId = $deploy['ID'];
            $this->productDeployService->updateDeployStepRemoveDeploy($deployId);
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveLog(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $deployId = empty($row['deployId']) ? null : $row['deployId'] ;
            $customizationId = empty($row['customizationId']) ? null : $row['customizationId'] ;
            $document = empty($row['DOCUMENT']) ? null : $row['DOCUMENT'];
            $this->productDeployService->saveLog($document, $deployId, null, $customizationId);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function getFullLog(Request\Filter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $logFileName = empty($conditions[0]) ? '' : $conditions[0]['value'];
        try {
            $logFileContents = $this->productDeployService->getFullLog($logFileName);
            $logFileContents = utf8_encode($logFileContents);
            $response->addDataSet(new DataSet('logFileContents', [$logFileContents]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findDeployMigrations(Request\Filter $request, Response $response) {
        try {
            $conditions  = $request->getFilterCriteria()->getConditions();
            $deployId    = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_ID', true);
            $attachments = $this->productDeployService->findDeployMigrations($deployId, $conditions);
            $response->addDataSet(new DataSet('deployMigrations', $attachments));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function downloadDeployMigration(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $attachmentId = $row['ID'];
            $this->productDeployService->downloadDeployMigration($attachmentId);
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function saveMigrationScript(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $deployId = empty($row['deployId']) ? null : $row['deployId'] ;
            $document = empty($row['DOCUMENT']) ? null : $row['DOCUMENT'];
            $type = empty($row['TYPE']) ? null : $row['TYPE'];
            $savedFiles = $this->productDeployService->saveLog($document, $deployId, $type);
            $response->addDataSet(new DataSet('savedFiles', $savedFiles));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function saveInitialMigrationScript(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $repositoryId   = $row['REPOSITORY_ID'];
            $deployId       = $row['DEPLOY_ID'];
            $fileNumber     = $row['FILE_NUMBER'];
            $lastFileNumber = $row['LAST_FILE_NUMBER'];
            $savedRegisters = $this->productDeployService->saveInitialMigrationScript($repositoryId, $deployId, $fileNumber, $lastFileNumber);
            $response->addDataSet(new DataSet('savedRegisters', $savedRegisters));
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findAvailableVersions(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $registers  = $this->productDeployService->findAvailableVersions($productId);
            $response->addDataSet(new DataSet('availableVersions', $registers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findDeployErrors(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $log  = Utilities::getValueInFilterDataSourceByName($conditions, 'LOG', true);
            $knowErrors  = $this->productDeployService->findDeployErrors($conditions, $log);
            $response->addDataSet(new DataSet('knowErrors', $knowErrors));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function validateTicket(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $ticket  = Utilities::getValueInFilterDataSourceByName($conditions, 'NRSEQCONTATO', true);
            $validate  = $this->productDeployService->validateTicket($ticket);
            $response->addDataSet(new DataSet('dataset', $validate));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findTicketMovDeskDeploy(Request\Filter $request, Response $response) {
        try {
            $validate  = $this->productDeployService->findTicketMovDeskDeploy();
            $response->addDataSet(new DataSet('findTicketMovDeskDeploy', $validate));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function removeUnappliedDeploy(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $deploy     = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY', true);
            $this->productDeployService->removeUnappliedDeploy($deploy);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getMigrationInfo(Request\Filter $request, Response $response) {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $migrationConfig = Utilities::getValueInFilterDataSourceByName($conditions, 'MIGRATION', true);
            $migrationInfo   = $this->productDeployService->getMigrationInfo($migrationConfig);
            $response->addDataSet(new DataSet('migrationInfo', $migrationInfo));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function findDeployUpdates(Request\Filter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $deployUpdates = $this->productDeployService->findDeployUpdates($conditions);
            $response->addDataSet(new DataSet('findDeployUpdates', $deployUpdates));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function findProductsByDeploy(Request\Filter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $deployId       = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_ID', true);
            $deployUpdates = $this->productDeployService->findProductsByDeploy($deployId, $conditions);
            $response->addDataSet(new DataSet('findProductsByDeploy', $deployUpdates));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findUpdatesBeforeDeploy(Request\Filter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $products = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCTS', true);
            $isClient = Utilities::getValueInFilterDataSourceByName($conditions, 'IS_CLIENT', true);
            $updates = $this->productDeployService->findUpdatesBeforeDeploy($products, $isClient);
            $response->addDataSet(new DataSet('findUpdatesBeforeDeploy', $updates));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function deployStep(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page       = $request->getFilterCriteria()->getPage();
            $pageSize   = $request->getFilterCriteria()->getPageSize();
            $id         = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_STEP_ID', true);
            $deployStep = $this->productDeployService->deployStep($id, $conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('deployStep', $deployStep));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function saveDeployStep(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $id  = $this->productDeployService->saveDeployStep($row);
            $response->addDataSet(new DataSet('savedDeployStep', array('ID' => $id)));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function removeDeployStep(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $id  = $row['ID'];
            $this->productDeployService->removeDeployStep($id);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function applyDeployStepInfo(Request\Filter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $deployStepId    = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_STEP_ID',    true);
            $isReapplyDeploy = Utilities::getValueInFilterDataSourceByName($conditions, 'IS_REAPPLY_DEPLOY', true);
            $serverInfo      = $this->productDeployService->applyDeployStepInfo($deployStepId, $isReapplyDeploy);
            $response->addDataSet(new DataSet('applyDeployStepInfo', $serverInfo));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function returnBkpDeployStepInfo(Request\Filter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $deployStepId    = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_STEP_ID',    true);
            $productServerId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_SERVER_ID', true);
            $serverInfo      = $this->productDeployService->returnBkpDeployStepInfo($deployStepId, $productServerId);
            $response->addDataSet(new DataSet('returnBkpDeployStepInfo', $serverInfo));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function newsDeployStepFilter(Request\Filter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $deployStepId    = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_STEP_ID',    true);
            $productServerId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_SERVER_ID', true);
            $newsFilter      = $this->productDeployService->getNewsDeployStepFilter($deployStepId, $productServerId);
            $response->addDataSet(new DataSet('newsDeployStepFilter', $newsFilter));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function validateCreateDeploy(Request\Filter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $productServerId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_SERVER_ID', true);
            $validation      = $this->productDeployService->validateCreateDeploy($productServerId);
            $response->addDataSet(new DataSet('validateCreateDeploy', array($validation)));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function validateDeployPermition(Request\Filter $request, Response $response)
    {
        try {
            $conditions      = $request->getFilterCriteria()->getConditions();
            $productId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $validation      = $this->productDeployService->validateDeployPermition($productId);
            $response->addDataSet(new DataSet('validateDeployPermition', array($validation)));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function getModulesDependencies(Request\Filter $request, Response $response)
    {
        try {
            $conditions     = $request->getFilterCriteria()->getConditions();
            $productModules = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_MODULES',  true);
            $serverId       = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_ID',        true);
            $serverBasePath = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_BASE_PATH', true);
            $modules        = $this->productDeployService->getModulesDependencies($productModules, $serverId, $serverBasePath);
            $response->addDataSet(new DataSet('getModulesDependencies', $modules));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function getDeployProducts(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $deployId   = Utilities::getValueInFilterDataSourceByName($conditions, 'DEPLOY_ID',  true);
            $products   = $this->productDeployService->getDeployProducts($deployId);
            $response->addDataSet(new DataSet('getDeployProducts', $products));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function findDeployCustomization(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page       = $request->getFilterCriteria()->getPage();
            $pageSize   = $request->getFilterCriteria()->getPageSize();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $deploys    = $this->productDeployService->findDeployCustomization($productId, $conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('findDeployCustomization', $deploys));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function createDeployCustomization(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $productId = $row['PRODUCT_ID'];
            $description = $row['DESCRIPTION'];
            $productServerId = $row['PRODUCT_SERVER_ID'];
            $deployCustomization = $this->productDeployService->createDeployCustomization($productId, $productServerId, $description);
            $response->addDataSet(new DataSet('createDeployCustomization', $deployCustomization));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function updateDeployCustomization(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $id = $row['ID'];
            $status = $row['STATUS'];
            $version = $row['VERSION'];
            $productId = $row['PRODUCT_ID'];
            $previousVersion = $row['PREVIOUS_VERSION'];
            $isUpdateSuccess = $row['IS_UPDATE_SUCCESS'];
            $migFullLog = $row['MIG_FULL_LOG'];
            $migErrorLog = $row['MIG_ERROR_LOG'];
            $this->productDeployService->updateDeployCustomization($productId, $id, $status, $version, $previousVersion, $isUpdateSuccess, $migFullLog, $migErrorLog);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}
